
# q-expansion of newform 2940.2.a.j, downloaded from the LMFDB on 16 November 2024.

# We generate the q-expansion using the Hecke eigenvalues a_p at the primes.
# Each a_p is given as a linear combination
# of the following basis for the coefficient ring.

def make_data():

    from sage.all import prod, floor, prime_powers, gcd, QQ, primes_first_n, next_prime, RR

    def discrete_log(elts, gens, mod):
        # algorithm 2.2, page 16 of https://arxiv.org/abs/0903.2785
        def table_gens(gens, mod):
            T = [1]
            n = len(gens)
            r = [None]*n
            s = [None]*n
            for i in range(n):
                beta = gens[i]
                r[i] = 1
                N = len(T)
                while beta not in T:
                    for Tj in T[:N]:
                        T.append((beta*Tj) % mod)
                    beta = (beta*gens[i]) % mod
                    r[i] += 1
                s[i] = T.index(beta)
            return T, r, s
        T, r, s = table_gens(gens, mod)
        n = len(gens)
        N = [ prod(r[:j]) for j in range(n) ]
        Z = lambda s: [ (floor(s/N[j]) % r[j]) for j in range(n)]
        return [Z(T.index(elt % mod)) for elt in elts]
    def extend_multiplicatively(an):
        for pp in prime_powers(len(an)-1):
            for k in range(1, (len(an) - 1)//pp + 1):
                if gcd(k, pp) == 1:
                    an[pp*k] = an[pp]*an[k]
    from sage.all import PolynomialRing, NumberField
    R = PolynomialRing(QQ, "x")
    f = R(poly_data)
    K = NumberField(f, "a")
    betas = [K.gens()[0]**i for i in range(len(poly_data))]
    convert_elt_to_field = lambda elt: sum(c*beta for c, beta in zip(elt, betas))
    # convert aps to K elements
    primes = primes_first_n(len(aps_data))
    good_primes = [p for p in primes if not p.divides(level)]
    aps = map(convert_elt_to_field, aps_data)
    if not hecke_ring_character_values:
        # trivial character
        char_values = dict(zip(good_primes, [1]*len(good_primes)))
    else:
        gens = [elt[0] for elt in hecke_ring_character_values]
        gens_values = [convert_elt_to_field(elt[1]) for elt in hecke_ring_character_values]
        char_values = dict([(
            p,prod(g**k for g, k in zip(gens_values, elt)))
            for p, elt in zip(good_primes, discrete_log(good_primes, gens, level))
            ])
    an_list_bound = next_prime(primes[-1])
    an = [0]*an_list_bound
    an[1] = 1
    
    from sage.all import PowerSeriesRing
    PS = PowerSeriesRing(K, "q")
    for p, ap in zip(primes, aps):
        if p.divides(level):
            euler_factor = [1, -ap]
        else:
            euler_factor = [1, -ap, p**(weight - 1) * char_values[p]]
        k = RR(an_list_bound).log(p).floor() + 1
        foo = (1/PS(euler_factor)).padded_list(k)
        for i in range(1, k):
            an[p**i] = foo[i]
    extend_multiplicatively(an)
    return PS(an)
level = 2940
weight = 2
poly_data = [0, 1]

# The basis for the coefficient ring is just the power basis
# in the root of the defining polynomial above.
hecke_ring_character_values = None
aps_data = [[0], [1], [1], [0], [-6], [0], [-6], [-4], [6], [-2], [8], [-2], [-10], [-12], [-8], [-2], [-4], [8], [-16], [-10], [0], [4], [4], [6], [8], [10], [12], [-2], [-6], [6], [4], [-12], [-14], [-16], [18], [-8], [4], [-16], [24], [-6], [-10], [-20], [-2], [-2], [-18], [12], [12], [16], [-4], [28], [2], [-22], [-4], [4], [22], [-2], [-2], [-24], [10], [-18], [16], [-14], [-8], [-24], [20], [-2], [-20], [22], [-22], [32], [-14], [-18], [8], [22], [28], [-8], [30], [28], [14], [20], [-12], [26], [-18], [-36], [28], [-6], [26], [2], [-38], [20], [36], [-24], [0], [-10], [-20], [24], [-26], [10], [4], [-30], [-24], [38], [-12], [-10], [32], [4], [44], [2], [-10], [-48], [-16], [-6], [18], [-40], [-4], [-18], [-40], [8], [-46], [18], [24], [34], [-46], [18], [-8], [6], [2], [8], [4], [8], [8], [42], [-32], [26], [-18], [40], [-14], [-52], [18], [-30], [-16], [42], [-36], [-18], [4], [40], [44], [-18], [28], [-50], [-14], [-6], [44], [32], [-4], [6], [-56], [-18], [36], [-10], [6], [30], [-36], [-52], [-54], [48], [32], [-12], [-2], [42], [18], [8], [-2], [-54], [4], [-30], [-32], [-42], [36], [-48], [-20], [12], [-58], [42], [46], [42], [-2], [52], [10], [-48], [56], [6], [-12], [18], [22], [-46], [-22], [46], [30], [-8], [-14], [-28], [40], [28], [4], [58], [-20], [22], [26], [-4], [14], [-26], [-16], [-12], [-56], [-48], [0], [10], [54], [-6], [-62], [-40], [50], [32], [-68], [6], [-42], [-54], [-52], [14], [18], [-68], [48], [-70], [-56], [0], [-68], [-66], [-50], [8], [50], [-32], [-8], [2], [-58], [-24], [68], [12], [16], [22], [-18], [10], [-62], [-4], [42], [-54], [-10], [-16], [18], [-28], [-44], [-22], [4], [-20], [18], [-56], [-6], [-6], [-56], [26], [48], [-4], [-56], [56], [-16], [-68], [-66], [-46], [62], [36], [-24], [28], [48], [16], [-44], [54], [-54], [-18], [24], [38], [18], [12], [18], [36], [-82], [-62], [28], [6], [-36], [-28], [-44], [50], [60], [70], [32], [54], [-70], [68], [54], [-14], [56], [-42], [-34], [4], [-18], [-12], [-36], [58], [-8], [-26], [-28], [-22], [-30], [-64], [-18], [20], [-64], [-56], [-22], [74], [-74], [-14], [-16], [-4], [-32], [4], [-46], [62], [60], [-20], [86], [42], [-18], [-24], [-46], [58], [-4], [-44], [72], [-14], [36], [-58], [-62], [64], [-26], [-86], [56], [-28], [-42], [-46], [-58], [-42], [-6], [66], [52], [-6], [22], [44], [78], [-10], [-60], [66], [-6], [-40], [-46], [-60], [58], [36], [-14], [8], [-22], [14], [88], [14], [-40], [24], [56], [56], [40], [-96], [50], [-90], [10], [-32], [30], [-34], [52], [50], [48], [-90], [-56], [-26], [-100], [-18], [30], [-28], [-62], [42], [-20], [12], [92], [-6], [66], [20], [90], [54], [42], [-28], [-54], [24], [-18], [-36], [58], [-60], [-68], [-86], [-30], [30], [-80], [64]]
